use crate::database;
use crate::usimp::*;
use std::collections::HashMap;
use std::sync::Arc;
use tokio::sync::{mpsc, Mutex};

static mut ROOMS: Option<Arc<Mutex<HashMap<String, Vec<mpsc::Sender<Event>>>>>> = None;
static mut ACCOUNTS: Option<Arc<Mutex<HashMap<String, Vec<mpsc::Sender<Event>>>>>> = None;

pub fn init() {
    unsafe {
        ROOMS = Some(Arc::new(Mutex::new(HashMap::new())));
        ACCOUNTS = Some(Arc::new(Mutex::new(HashMap::new())));
    }
}

pub async fn subscribe_account(account: &Account) -> mpsc::Receiver<Event> {
    let (tx, rx) = mpsc::channel::<Event>(64);
    unsafe {
        let mut acc = ACCOUNTS.as_ref().unwrap().lock().await;
        match acc.get_mut(account.id.as_str()) {
            Some(vec) => {
                vec.push(tx);
            }
            None => {
                acc.insert(account.id.clone(), vec![tx]);
            }
        }
    }
    rx
}

pub async fn push(room_id: &str, event: Event) -> Result<(), Error> {
    let backend = database::client().await?;
    let accounts = match backend {
        database::Client::Postgres(client) => {
            let res = client
                .query(
                    "SELECT account_id \
                FROM members \
                WHERE room_id = $1;",
                    &[&room_id],
                )
                .await?;
            let mut acc: Vec<String> = Vec::new();
            for row in res {
                acc.push(row.get(0));
            }
            acc
        }
    };

    unsafe {
        let mut rooms = ROOMS.as_ref().unwrap().lock().await;
        if let Some(rooms) = rooms.get_mut(room_id) {
            for tx in rooms {
                let _res = tx.send(event.clone()).await;
            }
        }
    }

    for account in accounts {
        unsafe {
            let mut accounts = ACCOUNTS.as_ref().unwrap().lock().await;
            if let Some(acc) = accounts.get_mut(account.as_str()) {
                for tx in acc {
                    let _res = tx.send(event.clone()).await;
                }
            }
        }
    }

    Ok(())
}
